@@ -1,7 +1,7 @@
 /*
  * coap_io.h -- Default network I/O functions for libcoap
  *
- * Copyright (C) 2012-2013,2023 Olaf Bergmann <bergmann@tzi.org>
+ * Copyright (C) 2012-2013,2024 Olaf Bergmann <bergmann@tzi.org>
  *
  * SPDX-License-Identifier: BSD-2-Clause
  *
@@ -59,13 +59,6 @@ typedef struct coap_addr_tuple_t {
 
 const char *coap_socket_strerror(void);
 
-/**
- * Check whether TCP is available.
- *
- * @return @c 1 if support for TCP is enabled, or @c 0 otherwise.
- */
-int coap_tcp_is_supported(void);
-
 typedef enum {
   COAP_NACK_TOO_MANY_RETRIES,
   COAP_NACK_NOT_DELIVERABLE,
