@@ -1,6 +1,6 @@
 /* coap_async.c -- state management for asynchronous messages
  *
- * Copyright (C) 2010,2011,2021-2023 Olaf Bergmann <bergmann@tzi.org>
+ * Copyright (C) 2010,2011,2021-2024 Olaf Bergmann <bergmann@tzi.org>
  *
  * SPDX-License-Identifier: BSD-2-Clause
  *
@@ -13,7 +13,7 @@
  * @brief State handling for asynchronous messages
  */
 
-#include "coap3/coap_internal.h"
+#include "coap3/coap_libcoap_build.h"
 
 #if COAP_ASYNC_SUPPORT
 #include <stdio.h>
@@ -35,13 +35,25 @@ coap_async_is_supported(void) {
   return 1;
 }
 
-coap_async_t *
+COAP_API coap_async_t *
 coap_register_async(coap_session_t *session,
                     const coap_pdu_t *request, coap_tick_t delay) {
+  coap_async_t *async;
+
+  coap_lock_lock(session->context, return NULL);
+  async = coap_register_async_lkd(session, request, delay);
+  coap_lock_unlock(session->context);
+  return async;
+}
+
+coap_async_t *
+coap_register_async_lkd(coap_session_t *session,
+                        const coap_pdu_t *request, coap_tick_t delay) {
   coap_async_t *s;
   size_t len;
   const uint8_t *data;
 
+  coap_lock_check_locked(session->context);
   if (!COAP_PDU_IS_REQUEST(request))
     return NULL;
 
@@ -77,10 +89,10 @@ coap_register_async(coap_session_t *session,
   LL_PREPEND(session->context->async_state, s);
 
   /* Note that this generates a new MID */
-  s->pdu = coap_pdu_duplicate(request, session, request->actual_token.length,
-                              request->actual_token.s, NULL);
+  s->pdu = coap_pdu_duplicate_lkd(request, session, request->actual_token.length,
+                                  request->actual_token.s, NULL);
   if (s->pdu == NULL) {
-    coap_free_async(session, s);
+    coap_free_async_lkd(session, s);
     coap_log_crit("coap_register_async: insufficient memory\n");
     return NULL;
   }
@@ -89,38 +101,49 @@ coap_register_async(coap_session_t *session,
     coap_add_data(s->pdu, len, data);
   }
 
-  s->session = coap_session_reference(session);
+  s->session = coap_session_reference_lkd(session);
 
-  coap_async_set_delay(s, delay);
+  coap_async_set_delay_lkd(s, delay);
 
   return s;
 }
 
-void
+COAP_API void
 coap_async_trigger(coap_async_t *async) {
+  coap_lock_lock(async->session->context, return);
+  coap_async_trigger_lkd(async);
+  coap_lock_unlock(async->session->context);
+}
+
+void
+coap_async_trigger_lkd(coap_async_t *async) {
   assert(async != NULL);
+  coap_lock_check_locked(async->session->context);
   coap_ticks(&async->delay);
 
   coap_log_debug("   %s: Async request triggered\n",
                  coap_session_str(async->session));
-#ifdef COAP_EPOLL_SUPPORT
-  coap_update_epoll_timer(async->session->context, 0);
-#endif /* COAP_EPOLL_SUPPORT */
+  coap_update_io_timer(async->session->context, 0);
 }
 
+COAP_API void
+coap_async_set_delay(coap_async_t *async, coap_tick_t delay) {
+  coap_lock_lock(async->session->context, return);
+  coap_async_set_delay_lkd(async, delay);
+  coap_lock_unlock(async->session->context);
+}
 
 void
-coap_async_set_delay(coap_async_t *async, coap_tick_t delay) {
+coap_async_set_delay_lkd(coap_async_t *async, coap_tick_t delay) {
   coap_tick_t now;
 
+  coap_lock_check_locked(async->session->context);
   assert(async != NULL);
   coap_ticks(&now);
 
   if (delay) {
     async->delay = now + delay;
-#ifdef COAP_EPOLL_SUPPORT
-    coap_update_epoll_timer(async->session->context, delay);
-#endif /* COAP_EPOLL_SUPPORT */
+    coap_update_io_timer(async->session->context, delay);
     coap_log_debug("   %s: Async request delayed for %u.%03u secs\n",
                    coap_session_str(async->session),
                    (unsigned int)(delay / COAP_TICKS_PER_SECOND),
@@ -133,10 +156,21 @@ coap_async_set_delay(coap_async_t *async, coap_tick_t delay) {
   }
 }
 
-coap_async_t *
+COAP_API coap_async_t *
 coap_find_async(coap_session_t *session, coap_bin_const_t token) {
   coap_async_t *tmp;
 
+  coap_lock_lock(session->context, return NULL);
+  tmp = coap_find_async_lkd(session, token);
+  coap_lock_unlock(session->context);
+  return tmp;
+}
+
+coap_async_t *
+coap_find_async_lkd(coap_session_t *session, coap_bin_const_t token) {
+  coap_async_t *tmp;
+
+  coap_lock_check_locked(session->context);
   SEARCH_PAIR(session->context->async_state, tmp,
               session, session,
               pdu->actual_token.length, token.length,
@@ -149,7 +183,7 @@ coap_free_async_sub(coap_context_t *context, coap_async_t *s) {
   if (s) {
     LL_DELETE(context->async_state,s);
     if (s->session) {
-      coap_session_release(s->session);
+      coap_session_release_lkd(s->session);
     }
     if (s->pdu) {
       coap_delete_pdu(s->pdu);
@@ -159,9 +193,16 @@ coap_free_async_sub(coap_context_t *context, coap_async_t *s) {
   }
 }
 
+COAP_API void
+coap_free_async(coap_session_t *session, coap_async_t *async) {
+  coap_lock_lock(session->context, return);
+  coap_free_async_lkd(session, async);
+  coap_lock_unlock(session->context);
+}
+
 void
-coap_free_async(coap_session_t *session, coap_async_t *s) {
-  coap_free_async_sub(session->context, s);
+coap_free_async_lkd(coap_session_t *session, coap_async_t *async) {
+  coap_free_async_sub(session->context, async);
 }
 
 void
