@@ -1,6 +1,6 @@
 /* coap_io.c -- Default network I/O functions for libcoap
  *
- * Copyright (C) 2012,2014,2016-2023 Olaf Bergmann <bergmann@tzi.org> and others
+ * Copyright (C) 2012,2014,2016-2024 Olaf Bergmann <bergmann@tzi.org> and others
  *
  * SPDX-License-Identifier: BSD-2-Clause
  *
@@ -13,7 +13,7 @@
  * @brief Network I/O functions
  */
 
-#include "coap3/coap_internal.h"
+#include "coap3/coap_libcoap_build.h"
 
 #ifdef HAVE_STDIO_H
 #  include <stdio.h>
@@ -54,7 +54,7 @@
 #endif
 #endif /* COAP_EPOLL_SUPPORT */
 
-#if !defined(WITH_CONTIKI) && !defined(RIOT_VERSION) && !(WITH_LWIP)
+#if !defined(WITH_CONTIKI) && !defined(RIOT_VERSION) && !defined(WITH_LWIP)
 /* define generic PKTINFO for IPv4 */
 #if defined(IP_PKTINFO)
 #  define GEN_IP_PKTINFO IP_PKTINFO
@@ -72,7 +72,7 @@
 #else
 #  error "Need IPV6_PKTINFO or IPV6_RECVPKTINFO to request ancillary data from OS."
 #endif /* IPV6_RECVPKTINFO */
-#endif /* !(WITH_CONTIKI || RIOT_VERSION) */
+#endif /* ! WITH_CONTIKI && ! RIOT_VERSION && ! WITH_LWIP */
 
 #if COAP_SERVER_SUPPORT
 coap_endpoint_t *
@@ -86,8 +86,19 @@ coap_mfree_endpoint(coap_endpoint_t *ep) {
 }
 #endif /* COAP_SERVER_SUPPORT */
 
-#if !defined(WITH_CONTIKI) && !defined(WITH_LWIP)
+#if defined(__MINGW32__)
+#if(_WIN32_WINNT >= 0x0600)
+#define CMSG_FIRSTHDR WSA_CMSG_FIRSTHDR
+#define CMSG_NXTHDR WSA_CMSG_NXTHDR
+#define CMSG_LEN WSA_CMSG_LEN
+#define CMSG_SPACE WSA_CMSG_SPACE
+#define cmsghdr _WSACMSGHDR
+#endif /* (_WIN32_WINNT>=0x0600) */
+#endif /* defined(__MINGW32__) */
 
+#if !defined(WITH_CONTIKI) && !defined(WITH_LWIP) && !defined(RIOT_VERSION)
+
+#if COAP_SERVER_SUPPORT
 int
 coap_socket_bind_udp(coap_socket_t *sock,
                      const coap_address_t *listen_addr,
@@ -112,10 +123,11 @@ coap_socket_bind_udp(coap_socket_t *sock,
   }
 #ifndef RIOT_VERSION
 #ifdef _WIN32
-  if (ioctlsocket(sock->fd, FIONBIO, &u_on) == COAP_SOCKET_ERROR) {
+  if (ioctlsocket(sock->fd, FIONBIO, &u_on) == COAP_SOCKET_ERROR)
 #else
-  if (ioctl(sock->fd, FIONBIO, &on) == COAP_SOCKET_ERROR) {
+  if (ioctl(sock->fd, FIONBIO, &on) == COAP_SOCKET_ERROR)
 #endif
+  {
     coap_log_warn("coap_socket_bind_udp: ioctl FIONBIO: %s\n", coap_socket_strerror());
   }
 
@@ -158,12 +170,12 @@ coap_socket_bind_udp(coap_socket_t *sock,
     coap_log_alert("coap_socket_bind_udp: unsupported sa_family\n");
     break;
   }
-#else /* ! RIOT_VERSION */
+#else /* RIOT_VERSION */
   if (setsockopt(sock->fd, SOL_SOCKET, SO_RCVTIMEO, OPTVAL_T(&timeout),
                  (socklen_t)sizeof(timeout)) == COAP_SOCKET_ERROR)
     coap_log_alert("coap_socket_bind_udp: setsockopt SO_RCVTIMEO: %s\n",
                    coap_socket_strerror());
-#endif /* ! RIOT_VERSION */
+#endif /* RIOT_VERSION */
 
   if (bind(sock->fd, &listen_addr->addr.sa,
 #if COAP_IPV4_SUPPORT
@@ -197,6 +209,7 @@ error:
   coap_socket_close(sock);
   return 0;
 }
+#endif /* COAP_SERVER_SUPPORT */
 
 #if COAP_CLIENT_SUPPORT
 int
@@ -206,21 +219,15 @@ coap_socket_connect_udp(coap_socket_t *sock,
                         int default_port,
                         coap_address_t *local_addr,
                         coap_address_t *remote_addr) {
-#ifndef RIOT_VERSION
   int on = 1;
 #if COAP_IPV6_SUPPORT
   int off = 0;
 #endif /* COAP_IPV6_SUPPORT */
-#else /* ! RIOT_VERSION */
-  struct timeval timeout = {0, 0};
-#endif /* ! RIOT_VERSION */
 #ifdef _WIN32
   u_long u_on = 1;
 #endif
   coap_address_t connect_addr;
-#if !defined(RIOT_VERSION)
   int is_mcast = coap_is_mcast(server);
-#endif /* !defined(RIOT_VERSION) */
   coap_address_copy(&connect_addr, server);
 
   sock->flags &= ~(COAP_SOCKET_CONNECTED | COAP_SOCKET_MULTICAST);
@@ -232,7 +239,6 @@ coap_socket_connect_udp(coap_socket_t *sock,
     goto error;
   }
 
-#ifndef RIOT_VERSION
 #ifdef _WIN32
   if (ioctlsocket(sock->fd, FIONBIO, &u_on) == COAP_SOCKET_ERROR)
 #else
@@ -242,7 +248,6 @@ coap_socket_connect_udp(coap_socket_t *sock,
     coap_log_warn("coap_socket_connect_udp: ioctl FIONBIO: %s\n",
                   coap_socket_strerror());
   }
-#endif /* RIOT_VERSION */
 
   switch (connect_addr.addr.sa.sa_family) {
 #if COAP_IPV4_SUPPORT
@@ -255,13 +260,11 @@ coap_socket_connect_udp(coap_socket_t *sock,
   case AF_INET6:
     if (connect_addr.addr.sin6.sin6_port == 0)
       connect_addr.addr.sin6.sin6_port = htons(default_port);
-#ifndef RIOT_VERSION
     /* Configure the socket as dual-stacked */
     if (setsockopt(sock->fd, IPPROTO_IPV6, IPV6_V6ONLY, OPTVAL_T(&off),
                    sizeof(off)) == COAP_SOCKET_ERROR)
       coap_log_warn("coap_socket_connect_udp: setsockopt IPV6_V6ONLY: %s\n",
                     coap_socket_strerror());
-#endif /* RIOT_VERSION */
 #endif /* COAP_IPV6_SUPPORT */
     break;
 #if COAP_AF_UNIX_SUPPORT
@@ -280,11 +283,9 @@ coap_socket_connect_udp(coap_socket_t *sock,
                     "remote address family\n");
       goto error;
     }
-#ifndef RIOT_VERSION
     if (setsockopt(sock->fd, SOL_SOCKET, SO_REUSEADDR, OPTVAL_T(&on), sizeof(on)) == COAP_SOCKET_ERROR)
       coap_log_warn("coap_socket_connect_udp: setsockopt SO_REUSEADDR: %s\n",
                     coap_socket_strerror());
-#endif /* RIOT_VERSION */
     if (bind(sock->fd, &local_if->addr.sa,
 #if COAP_IPV4_SUPPORT
              local_if->addr.sa.sa_family == AF_INET ?
@@ -304,7 +305,6 @@ coap_socket_connect_udp(coap_socket_t *sock,
   }
 
   /* special treatment for sockets that are used for multicast communication */
-#if !defined(RIOT_VERSION)
   if (is_mcast) {
     if (!(local_if && local_if->addr.sa.sa_family)) {
       /* Bind to a (unused) port to simplify logging */
@@ -337,31 +337,6 @@ coap_socket_connect_udp(coap_socket_t *sock,
                     coap_socket_strerror());
     return 1;
   }
-#else /* defined(RIOT_VERSION) */
-  if (!(local_if && local_if->addr.sa.sa_family)) {
-    /* Bind to a (unused) port to simplify logging */
-    coap_address_t bind_addr;
-
-    coap_address_init(&bind_addr);
-    bind_addr.addr.sa.sa_family = connect_addr.addr.sa.sa_family;
-#if COAP_IPV6_SUPPORT
-    if (bind_addr.addr.sa.sa_family == AF_INET6)
-      bind_addr.addr.sin6.sin6_scope_id = connect_addr.addr.sin6.sin6_scope_id;
-#endif /* COAP_IPV6_SUPPORT */
-    if (bind(sock->fd, &bind_addr.addr.sa,
-             bind_addr.addr.sa.sa_family == AF_INET ?
-             (socklen_t)sizeof(struct sockaddr_in) :
-             (socklen_t)bind_addr.size) == COAP_SOCKET_ERROR) {
-      coap_log_warn("coap_socket_connect_udp: bind: %s\n",
-                    coap_socket_strerror());
-      goto error;
-    }
-  }
-  if (setsockopt(sock->fd, SOL_SOCKET, SO_RCVTIMEO, OPTVAL_T(&timeout),
-                 (socklen_t)sizeof(timeout)) == COAP_SOCKET_ERROR)
-    coap_log_alert("coap_socket_bind_udp: setsockopt SO_RCVTIMEO: %s\n",
-                   coap_socket_strerror());
-#endif /* defined(RIOT_VERSION) */
 
   if (connect(sock->fd, &connect_addr.addr.sa, connect_addr.size) == COAP_SOCKET_ERROR) {
 #if COAP_AF_UNIX_SUPPORT
@@ -520,9 +495,14 @@ coap_epoll_ctl_mod(coap_socket_t *sock,
                  coap_socket_strerror(), errno);
   }
 }
+#endif /* COAP_EPOLL_SUPPORT */
 
+#endif /* ! WITH_CONTIKI && ! WITH_LWIP && ! RIOT_VERSION*/
+
+#ifndef WITH_CONTIKI
 void
-coap_update_epoll_timer(coap_context_t *context, coap_tick_t delay) {
+coap_update_io_timer(coap_context_t *context, coap_tick_t delay) {
+#if COAP_EPOLL_SUPPORT
   if (context->eptimerfd != -1) {
     coap_tick_t now;
 
@@ -543,7 +523,7 @@ coap_update_epoll_timer(coap_context_t *context, coap_tick_t delay) {
       ret = timerfd_settime(context->eptimerfd, 0, &new_value, NULL);
       if (ret == -1) {
         coap_log_err("%s: timerfd_settime failed: %s (%d)\n",
-                     "coap_resource_notify_observers",
+                     "coap_update_io_timer",
                      coap_socket_strerror(), errno);
       }
 #ifdef COAP_DEBUG_WAKEUP_TIMES
@@ -554,9 +534,14 @@ coap_update_epoll_timer(coap_context_t *context, coap_tick_t delay) {
 #endif /* COAP_DEBUG_WAKEUP_TIMES */
     }
   }
-}
-
+#else /* COAP_EPOLL_SUPPORT */
+  (void)context;
+  (void)delay;
 #endif /* COAP_EPOLL_SUPPORT */
+}
+#endif /* ! WITH_CONTIKI */
+
+#if !defined(WITH_CONTIKI) && !defined(WITH_LWIP) && !defined(RIOT_VERSION)
 
 #ifdef _WIN32
 static void
@@ -725,14 +710,14 @@ coap_socket_read(coap_socket_t *sock, uint8_t *data, size_t data_len) {
   return r;
 }
 
-#endif /* ! WITH_CONTIKI && ! WITH_LWIP */
+#endif /* ! WITH_CONTIKI && ! WITH_LWIP && ! RIOT_VERSION */
 
 #if !defined(WITH_LWIP)
 #if (!defined(WITH_CONTIKI)) != ( defined(HAVE_NETINET_IN_H) || defined(HAVE_WS2TCPIP_H) )
 /* define struct in6_pktinfo and struct in_pktinfo if not available
    FIXME: check with configure
 */
-#if !defined(__MINGW32__)
+#if !defined(__MINGW32__) && !defined(RIOT_VERSION)
 struct in6_pktinfo {
   struct in6_addr ipi6_addr;        /* src/dst IPv6 address */
   unsigned int ipi6_ifindex;        /* send/recv interface index */
@@ -759,7 +744,9 @@ struct in_pktinfo {
 
 #if defined(_WIN32)
 #include <mswsock.h>
-#if !defined(__MINGW32__)
+#if defined(__MINGW32__)
+static __thread LPFN_WSARECVMSG lpWSARecvMsg = NULL;
+#else /* ! __MINGW32__ */
 static __declspec(thread) LPFN_WSARECVMSG lpWSARecvMsg = NULL;
 #endif /* ! __MINGW32__ */
 /* Map struct WSABUF fields to their posix counterpart */
@@ -784,21 +771,64 @@ static __declspec(thread) LPFN_WSARECVMSG lpWSARecvMsg = NULL;
 #define iov_len_t size_t
 #endif
 
-#if defined(_CYGWIN_ENV)
+#if defined(_CYGWIN_ENV) || defined(__QNXNTO__)
 #define ipi_spec_dst ipi_addr
 #endif
 
 #if !defined(RIOT_VERSION) && !defined(WITH_LWIP) && !defined(WITH_CONTIKI)
+#if COAP_CLIENT_SUPPORT
+static uint32_t cid_track_counter;
+
+static void
+coap_test_cid_tuple_change(coap_session_t *session) {
+  if (session->type == COAP_SESSION_TYPE_CLIENT &&
+      session->negotiated_cid &&
+      session->state == COAP_SESSION_STATE_ESTABLISHED &&
+      session->proto == COAP_PROTO_DTLS && session->context->testing_cids) {
+    if ((++cid_track_counter) % session->context->testing_cids == 0) {
+      coap_address_t local_if = session->addr_info.local;
+      uint16_t port = coap_address_get_port(&local_if);
+
+      port++;
+      coap_address_set_port(&local_if, port);
+
+      coap_socket_close(&session->sock);
+      session->sock.session = session;
+      if (!coap_socket_connect_udp(&session->sock, &local_if, &session->addr_info.remote,
+                                   port,
+                                   &session->addr_info.local,
+                                   &session->addr_info.remote)) {
+        coap_log_err("Tuple change for CID failed\n");
+        return;
+#ifdef COAP_EPOLL_SUPPORT
+      } else {
+        coap_epoll_ctl_add(&session->sock,
+                           EPOLLIN |
+                           ((session->sock.flags & COAP_SOCKET_WANT_CONNECT) ?
+                            EPOLLOUT : 0),
+                           __func__);
+#endif /* COAP_EPOLL_SUPPORT */
+      }
+      session->sock.flags |= COAP_SOCKET_NOT_EMPTY | COAP_SOCKET_WANT_READ | COAP_SOCKET_BOUND;
+    }
+  }
+}
+#endif /* COAP_CLIENT_SUPPORT */
+
 /*
  * dgram
  * return +ve Number of bytes written.
  *         -1 Error error in errno).
  */
 ssize_t
-coap_socket_send(coap_socket_t *sock, const coap_session_t *session,
+coap_socket_send(coap_socket_t *sock, coap_session_t *session,
                  const uint8_t *data, size_t datalen) {
   ssize_t bytes_written = 0;
 
+#if COAP_CLIENT_SUPPORT
+  coap_test_cid_tuple_change(session);
+#endif /* COAP_CLIENT_SUPPORT */
+
   if (!coap_debug_send_packet()) {
     bytes_written = (ssize_t)datalen;
   } else if (sock->flags & COAP_SOCKET_CONNECTED) {
@@ -808,7 +838,7 @@ coap_socket_send(coap_socket_t *sock, const coap_session_t *session,
     bytes_written = send(sock->fd, data, datalen, 0);
 #endif
   } else {
-#if defined(_WIN32) && !defined(__MINGW32__)
+#if defined(_WIN32)
     DWORD dwNumberOfBytesSent = 0;
     int r;
 #endif /* _WIN32 && !__MINGW32__ */
@@ -945,7 +975,7 @@ coap_socket_send(coap_socket_t *sock, const coap_session_t *session,
     }
 #endif /* HAVE_STRUCT_CMSGHDR */
 
-#if defined(_WIN32) && !defined(__MINGW32__)
+#if defined(_WIN32)
     r = WSASendMsg(sock->fd, &mhdr, 0 /*dwFlags*/, &dwNumberOfBytesSent, NULL /*lpOverlapped*/,
                    NULL /*lpCompletionRoutine*/);
     if (r == 0)
@@ -980,7 +1010,7 @@ coap_packet_get_memmapped(coap_packet_t *packet, unsigned char **address, size_t
   *length = packet->length;
 }
 
-#if !defined(WITH_LWIP) && !defined(WITH_CONTIKI)
+#if !defined(WITH_LWIP) && !defined(WITH_CONTIKI) && !defined(RIOT_VERSION)
 /*
  * dgram
  * return +ve Number of bytes written.
@@ -1019,16 +1049,18 @@ coap_socket_recv(coap_socket_t *sock, coap_packet_t *packet) {
                       coap_socket_strerror());
         return -2;
       }
-      coap_log_warn("** %s: coap_socket_recv: %s\n",
-                    sock->session ?
-                    coap_session_str(sock->session) : "",
-                    coap_socket_strerror());
+      if (errno != EAGAIN) {
+        coap_log_warn("** %s: coap_socket_recv: %s\n",
+                      sock->session ?
+                      coap_session_str(sock->session) : "",
+                      coap_socket_strerror());
+      }
       goto error;
     } else if (len > 0) {
       packet->length = (size_t)len;
     }
   } else {
-#if defined(_WIN32) && !defined(__MINGW32__)
+#if defined(_WIN32)
     DWORD dwNumberOfBytesRecvd = 0;
     int r;
 #endif /* _WIN32 && !__MINGW32__ */
@@ -1039,7 +1071,11 @@ coap_socket_recv(coap_socket_t *sock, coap_packet_t *packet) {
     struct msghdr mhdr;
     struct iovec iov[1];
 
+#if defined(__MINGW32__)
+    iov[0].iov_base = (char *) packet->payload;
+#else
     iov[0].iov_base = packet->payload;
+#endif /* defined(__MINGW32__) */
     iov[0].iov_len = (iov_len_t)COAP_RXBUFFER_SIZE;
 
     memset(&mhdr, 0, sizeof(struct msghdr));
@@ -1083,14 +1119,6 @@ coap_socket_recv(coap_socket_t *sock, coap_packet_t *packet) {
     len = recvfrom(sock->fd, (void *)packet->payload, COAP_RXBUFFER_SIZE, 0,
                    &packet->addr_info.remote.addr.sa,
                    &packet->addr_info.remote.size);
-#if defined(RIOT_VERSION) && defined(COAP_SERVER_SUPPORT) && COAP_IPV6_SUPPORT
-    if (sock->endpoint &&
-        packet->addr_info.remote.addr.sa.sa_family == AF_INET6) {
-      packet->addr_info.remote.addr.sin6.sin6_scope_id =
-          sock->endpoint->bind_addr.addr.sin6.sin6_scope_id;
-      packet->addr_info.remote.addr.sin6.sin6_flowinfo = 0;
-    }
-#endif /* RIOT_VERSION && COAP_SERVER_SUPPORT && COAP_IPV6_SUPPORT */
 #endif /* ! HAVE_STRUCT_CMSGHDR */
 
     if (len < 0) {
@@ -1105,7 +1133,9 @@ coap_socket_recv(coap_socket_t *sock, coap_packet_t *packet) {
                       coap_socket_strerror());
         return 0;
       }
-      coap_log_warn("coap_socket_recv: %s\n", coap_socket_strerror());
+      if (errno != EAGAIN) {
+        coap_log_warn("coap_socket_recv: %s\n", coap_socket_strerror());
+      }
       goto error;
     } else {
 #ifdef HAVE_STRUCT_CMSGHDR
@@ -1197,14 +1227,6 @@ coap_socket_recv(coap_socket_t *sock, coap_packet_t *packet) {
         coap_log_debug("Cannot determine local port\n");
         goto error;
       }
-#if defined(RIOT_VERSION) && defined(COAP_SERVER_SUPPORT) && COAP_IPV6_SUPPORT
-      if (sock->endpoint &&
-          packet->addr_info.local.addr.sa.sa_family == AF_INET6) {
-        packet->addr_info.local.addr.sin6.sin6_scope_id =
-            sock->endpoint->bind_addr.addr.sin6.sin6_scope_id;
-        packet->addr_info.local.addr.sin6.sin6_flowinfo = 0;
-      }
-#endif /* RIOT_VERSION && COAP_SERVER_SUPPORT && COAP_IPV6_SUPPORT */
 #endif /* ! HAVE_STRUCT_CMSGHDR */
     }
   }
@@ -1214,10 +1236,20 @@ coap_socket_recv(coap_socket_t *sock, coap_packet_t *packet) {
 error:
   return -1;
 }
-#endif /* ! WITH_LWIP && ! WITH_CONTIKI */
+#endif /* ! WITH_LWIP && ! WITH_CONTIKI && ! RIOT_VERSION */
+
+COAP_API unsigned int
+coap_io_prepare_epoll(coap_context_t *ctx, coap_tick_t now) {
+  unsigned int ret;
+
+  coap_lock_lock(ctx, return 0);
+  ret = coap_io_prepare_epoll_lkd(ctx, now);
+  coap_lock_unlock(ctx);
+  return ret;
+}
 
 unsigned int
-coap_io_prepare_epoll(coap_context_t *ctx, coap_tick_t now) {
+coap_io_prepare_epoll_lkd(coap_context_t *ctx, coap_tick_t now) {
 #ifndef COAP_EPOLL_SUPPORT
   (void)ctx;
   (void)now;
@@ -1229,8 +1261,9 @@ coap_io_prepare_epoll(coap_context_t *ctx, coap_tick_t now) {
   unsigned int num_sockets;
   unsigned int timeout;
 
+  coap_lock_check_locked(ctx);
   /* Use the common logic */
-  timeout = coap_io_prepare_io(ctx, sockets, max_sockets, &num_sockets, now);
+  timeout = coap_io_prepare_io_lkd(ctx, sockets, max_sockets, &num_sockets, now);
   /* Save when the next expected I/O is to take place */
   ctx->next_timeout = timeout ? now + timeout : 0;
   if (ctx->eptimerfd != -1) {
@@ -1266,12 +1299,30 @@ coap_io_prepare_epoll(coap_context_t *ctx, coap_tick_t now) {
  * return  0 No i/o pending
  *       +ve millisecs to next i/o activity
  */
-unsigned int
+COAP_API unsigned int
 coap_io_prepare_io(coap_context_t *ctx,
                    coap_socket_t *sockets[],
                    unsigned int max_sockets,
                    unsigned int *num_sockets,
                    coap_tick_t now) {
+  unsigned int ret;
+
+  coap_lock_lock(ctx, return 0);
+  ret = coap_io_prepare_io_lkd(ctx, sockets, max_sockets, num_sockets, now);
+  coap_lock_unlock(ctx);
+  return ret;
+}
+
+/*
+ * return  0 No i/o pending
+ *       +ve millisecs to next i/o activity
+ */
+unsigned int
+coap_io_prepare_io_lkd(coap_context_t *ctx,
+                       coap_socket_t *sockets[],
+                       unsigned int max_sockets,
+                       unsigned int *num_sockets,
+                       coap_tick_t now) {
   coap_queue_t *nextpdu;
   coap_session_t *s, *rtmp;
   coap_tick_t timeout = 0;
@@ -1279,16 +1330,17 @@ coap_io_prepare_io(coap_context_t *ctx,
 #if COAP_SERVER_SUPPORT
   int check_dtls_timeouts = 0;
 #endif /* COAP_SERVER_SUPPORT */
-#if defined(COAP_EPOLL_SUPPORT) || defined(WITH_LWIP)
+#if defined(COAP_EPOLL_SUPPORT) || defined(WITH_LWIP) || defined(RIOT_VERSION)
   (void)sockets;
   (void)max_sockets;
-#endif /* COAP_EPOLL_SUPPORT || WITH_LWIP */
+#endif /* COAP_EPOLL_SUPPORT || WITH_LWIP || RIOT_VERSION*/
 
+  coap_lock_check_locked(ctx);
   *num_sockets = 0;
 
 #if COAP_SERVER_SUPPORT
   /* Check to see if we need to send off any Observe requests */
-  coap_check_notify(ctx);
+  coap_check_notify_lkd(ctx);
 
 #if COAP_ASYNC_SUPPORT
   /* Check to see if we need to send off any Async requests */
@@ -1325,6 +1377,12 @@ coap_io_prepare_io(coap_context_t *ctx,
 #endif /* COAP_SERVER_SUPPORT */
     }
   }
+#if COAP_PROXY_SUPPORT
+  if (coap_proxy_check_timeouts(ctx, now, &s_timeout)) {
+    if (timeout == 0 || s_timeout < timeout)
+      timeout = s_timeout;
+  }
+#endif /* COAP_PROXY_SUPPORT */
 #if COAP_SERVER_SUPPORT
   coap_endpoint_t *ep;
   coap_tick_t session_timeout;
@@ -1335,19 +1393,19 @@ coap_io_prepare_io(coap_context_t *ctx,
     session_timeout = COAP_DEFAULT_SESSION_TIMEOUT * COAP_TICKS_PER_SECOND;
 
   LL_FOREACH(ctx->endpoint, ep) {
-#if !defined(COAP_EPOLL_SUPPORT) && !defined(WITH_LWIP)
+#if !defined(COAP_EPOLL_SUPPORT) && !defined(WITH_LWIP) && !defined(RIOT_VERSION)
     if (ep->sock.flags & (COAP_SOCKET_WANT_READ | COAP_SOCKET_WANT_WRITE | COAP_SOCKET_WANT_ACCEPT)) {
       if (*num_sockets < max_sockets)
         sockets[(*num_sockets)++] = &ep->sock;
     }
-#endif /* ! COAP_EPOLL_SUPPORT i && ! WITH_LWIP */
+#endif /* ! COAP_EPOLL_SUPPORT && ! WITH_LWIP && ! RIOT_VERSION */
     SESSIONS_ITER_SAFE(ep->sessions, s, rtmp) {
       /* Check whether any idle server sessions should be released */
       if (s->type == COAP_SESSION_TYPE_SERVER && s->ref == 0 &&
           s->delayqueue == NULL &&
           (s->last_rx_tx + session_timeout <= now ||
            s->state == COAP_SESSION_STATE_NONE)) {
-        coap_handle_event(ctx, COAP_EVENT_SERVER_SESSION_DEL, s);
+        coap_handle_event_lkd(ctx, COAP_EVENT_SERVER_SESSION_DEL, s);
         coap_session_free(s);
       } else {
         if (s->type == COAP_SESSION_TYPE_SERVER && s->ref == 0 &&
@@ -1357,7 +1415,7 @@ coap_io_prepare_io(coap_context_t *ctx,
             timeout = s_timeout;
         }
         /* Make sure the session object is not deleted in any callbacks */
-        coap_session_reference(s);
+        coap_session_reference_lkd(s);
         /* Check any DTLS timeouts and expire if appropriate */
         if (check_dtls_timeouts && s->state == COAP_SESSION_STATE_HANDSHAKE &&
             s->proto == COAP_PROTO_DTLS && s->tls) {
@@ -1392,12 +1450,12 @@ coap_io_prepare_io(coap_context_t *ctx,
               timeout = s_timeout;
           }
         }
-#if !defined(COAP_EPOLL_SUPPORT) && !defined(WITH_LWIP)
+#if !defined(COAP_EPOLL_SUPPORT) && !defined(WITH_LWIP) && !defined(RIOT_VERSION)
         if (s->sock.flags & (COAP_SOCKET_WANT_READ|COAP_SOCKET_WANT_WRITE)) {
           if (*num_sockets < max_sockets)
             sockets[(*num_sockets)++] = &s->sock;
         }
-#endif /* ! COAP_EPOLL_SUPPORT && ! WITH_LWIP */
+#endif /* ! COAP_EPOLL_SUPPORT && ! WITH_LWIP && ! RIOT_VERSION */
 #if COAP_Q_BLOCK_SUPPORT
         /*
          * Check if any server large transmits have hit MAX_PAYLOAD and need
@@ -1410,7 +1468,7 @@ coap_io_prepare_io(coap_context_t *ctx,
         }
 #endif /* COAP_Q_BLOCK_SUPPORT */
 release_1:
-        coap_session_release(s);
+        coap_session_release_lkd(s);
       }
     }
   }
@@ -1422,11 +1480,11 @@ release_1:
         ctx->ping_timeout > 0) {
       if (s->last_rx_tx + ctx->ping_timeout * COAP_TICKS_PER_SECOND <= now) {
         /* Time to send a ping */
-        if ((s->last_ping_mid = coap_session_send_ping(s)) == COAP_INVALID_MID)
+        if ((s->last_ping_mid = coap_session_send_ping_lkd(s)) == COAP_INVALID_MID)
           /* Some issue - not safe to continue processing */
           continue;
         if (s->last_ping > 0 && s->last_pong < s->last_ping) {
-          coap_handle_event(s->context, COAP_EVENT_KEEPALIVE_FAILURE, s);
+          coap_handle_event_lkd(s->context, COAP_EVENT_KEEPALIVE_FAILURE, s);
         }
         s->last_rx_tx = now;
         s->last_ping = now;
@@ -1438,24 +1496,23 @@ release_1:
 
 #if !COAP_DISABLE_TCP
     if (s->type == COAP_SESSION_TYPE_CLIENT && COAP_PROTO_RELIABLE(s->proto) &&
-        s->state == COAP_SESSION_STATE_CSM && ctx->csm_timeout > 0) {
+        s->state == COAP_SESSION_STATE_CSM && ctx->csm_timeout_ms > 0) {
       if (s->csm_tx == 0) {
         s->csm_tx = now;
-      } else if (s->csm_tx + ctx->csm_timeout * COAP_TICKS_PER_SECOND <= now) {
-        /* Make sure the session object is not deleted in the callback */
-        coap_session_reference(s);
-        coap_session_disconnected(s, COAP_NACK_NOT_DELIVERABLE);
-        coap_session_release(s);
-        continue;
+        s_timeout = (ctx->csm_timeout_ms * COAP_TICKS_PER_SECOND) / 1000;
+      } else if (s->csm_tx + (ctx->csm_timeout_ms * COAP_TICKS_PER_SECOND) / 1000 <= now) {
+        /* timed out */
+        s_timeout = 0;
+      } else {
+        s_timeout = (s->csm_tx + (ctx->csm_timeout_ms * COAP_TICKS_PER_SECOND) / 1000) - now;
       }
-      s_timeout = (s->csm_tx + ctx->csm_timeout * COAP_TICKS_PER_SECOND) - now;
-      if (timeout == 0 || s_timeout < timeout)
+      if ((timeout == 0 || s_timeout < timeout) && s_timeout != 0)
         timeout = s_timeout;
     }
 #endif /* !COAP_DISABLE_TCP */
 
     /* Make sure the session object is not deleted in any callbacks */
-    coap_session_reference(s);
+    coap_session_reference_lkd(s);
     /* Check any DTLS timeouts and expire if appropriate */
     if (s->state == COAP_SESSION_STATE_HANDSHAKE &&
         s->proto == COAP_PROTO_DTLS && s->tls) {
@@ -1502,7 +1559,7 @@ release_1:
     }
 #endif /* COAP_Q_BLOCK_SUPPORT */
 
-#if !defined(COAP_EPOLL_SUPPORT) && !defined(WITHLWIP)
+#if !defined(COAP_EPOLL_SUPPORT) && !defined(WITH_LWIP) && !defined(RIOT_VERSION)
     assert(s->ref > 1);
     if (s->sock.flags & (COAP_SOCKET_WANT_READ |
                          COAP_SOCKET_WANT_WRITE |
@@ -1510,25 +1567,48 @@ release_1:
       if (*num_sockets < max_sockets)
         sockets[(*num_sockets)++] = &s->sock;
     }
-#endif /* ! COAP_EPOLL_SUPPORT && ! WITH_LWIP */
+#endif /* ! COAP_EPOLL_SUPPORT && ! WITH_LWIP && ! RIOT_VERSION */
 release_2:
-    coap_session_release(s);
+    coap_session_release_lkd(s);
   }
 #endif /* COAP_CLIENT_SUPPORT */
 
   return (unsigned int)((timeout * 1000 + COAP_TICKS_PER_SECOND - 1) / COAP_TICKS_PER_SECOND);
 }
 
-#if !defined(WITH_LWIP) && !defined(CONTIKI)
-int
+#if !defined(WITH_LWIP) && !defined(CONTIKI) && !defined(RIOT_VERSION)
+COAP_API int
 coap_io_process(coap_context_t *ctx, uint32_t timeout_ms) {
-  return coap_io_process_with_fds(ctx, timeout_ms, 0, NULL, NULL, NULL);
+  int ret;
+
+  coap_lock_lock(ctx, return 0);
+  ret = coap_io_process_lkd(ctx, timeout_ms);
+  coap_lock_unlock(ctx);
+  return ret;
 }
 
 int
+coap_io_process_lkd(coap_context_t *ctx, uint32_t timeout_ms) {
+  return coap_io_process_with_fds_lkd(ctx, timeout_ms, 0, NULL, NULL, NULL);
+}
+
+COAP_API int
 coap_io_process_with_fds(coap_context_t *ctx, uint32_t timeout_ms,
                          int enfds, fd_set *ereadfds, fd_set *ewritefds,
                          fd_set *eexceptfds) {
+  int ret;
+
+  coap_lock_lock(ctx, return 0);
+  ret = coap_io_process_with_fds_lkd(ctx, timeout_ms, enfds, ereadfds, ewritefds,
+                                     eexceptfds);
+  coap_lock_unlock(ctx);
+  return ret;
+}
+
+int
+coap_io_process_with_fds_lkd(coap_context_t *ctx, uint32_t timeout_ms,
+                             int enfds, fd_set *ereadfds, fd_set *ewritefds,
+                             fd_set *eexceptfds) {
   coap_fd_t nfds = 0;
   coap_tick_t before, now;
   unsigned int timeout;
@@ -1538,15 +1618,14 @@ coap_io_process_with_fds(coap_context_t *ctx, uint32_t timeout_ms,
   unsigned int i;
 #endif /* ! COAP_EPOLL_SUPPORT */
 
+  coap_lock_check_locked(ctx);
   coap_ticks(&before);
 
 #ifndef COAP_EPOLL_SUPPORT
 
-  timeout = coap_io_prepare_io(ctx, ctx->sockets,
-                               (sizeof(ctx->sockets) / sizeof(ctx->sockets[0])),
-                               &ctx->num_sockets, before);
-  if (timeout == 0 || timeout_ms < timeout)
-    timeout = timeout_ms;
+  timeout = coap_io_prepare_io_lkd(ctx, ctx->sockets,
+                                   (sizeof(ctx->sockets) / sizeof(ctx->sockets[0])),
+                                   &ctx->num_sockets, before);
 
   if (ereadfds) {
     ctx->readfds = *ereadfds;
@@ -1587,14 +1666,23 @@ coap_io_process_with_fds(coap_context_t *ctx, uint32_t timeout_ms,
     tv.tv_usec = 0;
     tv.tv_sec = 0;
     timeout = 1;
-  } else if (timeout > 0) {
+  } else if (timeout == 0 && timeout_ms == COAP_IO_WAIT) {
+    ;
+  } else {
+    if (timeout == 0 || (timeout_ms != COAP_IO_WAIT && timeout_ms < timeout))
+      timeout = timeout_ms;
     tv.tv_usec = (timeout % 1000) * 1000;
     tv.tv_sec = (long)(timeout / 1000);
   }
 
+  /* Unlock so that other threads can lock/update ctx */
+  coap_lock_unlock(ctx);
+
   result = select((int)nfds, &ctx->readfds, &ctx->writefds, &ctx->exceptfds,
                   timeout > 0 ? &tv : NULL);
 
+  coap_lock_lock(ctx, return -1);
+
   if (result < 0) {   /* error */
 #ifdef _WIN32
     coap_win_error_to_errno();
@@ -1615,6 +1703,12 @@ coap_io_process_with_fds(coap_context_t *ctx, uint32_t timeout_ms,
   }
 
   if (result > 0) {
+#if COAP_THREAD_SAFE
+    /* Need to refresh what is available to read / write etc. */
+    tv.tv_usec = 0;
+    tv.tv_sec = 0;
+    select((int)nfds, &ctx->readfds, &ctx->writefds, &ctx->exceptfds, &tv);
+#endif /* COAP_THREAD_SAFE */
     for (i = 0; i < ctx->num_sockets; i++) {
       if ((ctx->sockets[i]->flags & COAP_SOCKET_WANT_READ) &&
           FD_ISSET(ctx->sockets[i]->fd, &ctx->readfds))
@@ -1635,7 +1729,7 @@ coap_io_process_with_fds(coap_context_t *ctx, uint32_t timeout_ms,
   }
 
   coap_ticks(&now);
-  coap_io_do_io(ctx, now);
+  coap_io_do_io_lkd(ctx, now);
 
 #else /* COAP_EPOLL_SUPPORT */
   (void)ereadfds;
@@ -1643,36 +1737,63 @@ coap_io_process_with_fds(coap_context_t *ctx, uint32_t timeout_ms,
   (void)eexceptfds;
   (void)enfds;
 
-  timeout = coap_io_prepare_epoll(ctx, before);
-
-  if (timeout == 0 || timeout_ms < timeout)
-    timeout = timeout_ms;
+  timeout = coap_io_prepare_epoll_lkd(ctx, before);
 
   do {
     struct epoll_event events[COAP_MAX_EPOLL_EVENTS];
-    int etimeout = timeout;
+    int etimeout;
 
     /* Potentially adjust based on what the caller wants */
     if (timeout_ms == COAP_IO_NO_WAIT) {
+      /* Need to return immediately from epoll_wait() */
       etimeout = 0;
-    } else if (timeout == COAP_IO_WAIT) {
-      /* coap_io_prepare_epoll() returned 0 and timeout_ms COAP_IO_WAIT (0) */
+    } else if (timeout == 0 && timeout_ms == COAP_IO_WAIT) {
+      /*
+       * Nothing found in coap_io_prepare_epoll_lkd() and COAP_IO_WAIT set,
+       * so wait forever in epoll_wait().
+       */
       etimeout = -1;
-    } else if (etimeout < 0) {
-      /* epoll_wait cannot wait longer than this as int timeout parameter */
-      etimeout = INT_MAX;
+    } else {
+      etimeout = timeout;
+      if (timeout == 0 || (timeout_ms != COAP_IO_WAIT && timeout_ms < timeout))
+        etimeout = timeout_ms;
+      if (etimeout < 0) {
+        /*
+         * If timeout > INT_MAX, epoll_wait() cannot wait longer than this as
+         * it has int timeout parameter
+         */
+        etimeout = INT_MAX;
+      }
     }
 
+    /* Unlock so that other threads can lock/update ctx */
+    coap_lock_unlock(ctx);
+
     nfds = epoll_wait(ctx->epfd, events, COAP_MAX_EPOLL_EVENTS, etimeout);
     if (nfds < 0) {
       if (errno != EINTR) {
         coap_log_err("epoll_wait: unexpected error: %s (%d)\n",
                      coap_socket_strerror(), nfds);
       }
+      coap_lock_lock(ctx, return -1);
       break;
     }
 
-    coap_io_do_epoll(ctx, events, nfds);
+#if COAP_THREAD_SAFE
+    /* Need to refresh what is available to read / write etc. */
+    nfds = epoll_wait(ctx->epfd, events, COAP_MAX_EPOLL_EVENTS, 0);
+    if (nfds < 0) {
+      if (errno != EINTR) {
+        coap_log_err("epoll_wait: unexpected error: %s (%d)\n",
+                     coap_socket_strerror(), nfds);
+      }
+      coap_lock_lock(ctx, return -1);
+      break;
+    }
+#endif /* COAP_THREAD_SAFE */
+    coap_lock_lock(ctx, return -1);
+
+    coap_io_do_epoll_lkd(ctx, events, nfds);
 
     /*
      * reset to COAP_IO_NO_WAIT (which causes etimeout to become 0)
@@ -1698,14 +1819,24 @@ coap_io_process_with_fds(coap_context_t *ctx, uint32_t timeout_ms,
 
   return (int)(((now - before) * 1000) / COAP_TICKS_PER_SECOND);
 }
-#endif /* ! WITH_LWIP && ! WITH_CONTIKI */
+#endif /* ! WITH_LWIP && ! WITH_CONTIKI && ! RIOT_VERSION*/
+
+COAP_API int
+coap_io_pending(coap_context_t *context) {
+  int ret;
+
+  coap_lock_lock(context, return 0);
+  ret = coap_io_pending_lkd(context);
+  coap_lock_unlock(context);
+  return ret;
+}
 
 /*
  * return 1  I/O pending
  *        0  No I/O pending
  */
 int
-coap_io_pending(coap_context_t *context) {
+coap_io_pending_lkd(coap_context_t *context) {
   coap_session_t *s, *rtmp;
 #if COAP_SERVER_SUPPORT
   coap_endpoint_t *ep;
@@ -1713,7 +1844,8 @@ coap_io_pending(coap_context_t *context) {
 
   if (!context)
     return 0;
-  if (coap_io_process(context, COAP_IO_NO_WAIT) < 0)
+  coap_lock_check_locked(context);
+  if (coap_io_process_lkd(context, COAP_IO_NO_WAIT) < 0)
     return 0;
 
   if (context->sendqueue)
