@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2012,2014 Olaf Bergmann <bergmann@tzi.org>
- *               2014 chrysn <chrysn@fsfe.org>
- *               2022-2023 Jon Shallow <supjps-libcoap@jpshallow.com>
+ *               2014      chrysn <chrysn@fsfe.org>
+ *               2022-2024 Jon Shallow <supjps-libcoap@jpshallow.com>
  *
  * SPDX-License-Identifier: BSD-2-Clause
  *
@@ -14,7 +14,7 @@
  * @brief LwIP specific functions
  */
 
-#include "coap3/coap_internal.h"
+#include "coap3/coap_libcoap_build.h"
 #include <lwip/udp.h>
 #include <lwip/timeouts.h>
 #include <lwip/tcpip.h>
@@ -51,69 +51,93 @@ coap_lwip_set_input_wait_handler(coap_context_t *context,
   context->input_arg = input_arg;
 }
 
+#if NO_SYS == 0
+sys_sem_t coap_io_timeout_sem;
+#endif /* NO_SYS == 0 */
+
+void
+coap_io_lwip_init(void) {
+#if NO_SYS == 0
+  if (sys_sem_new(&coap_io_timeout_sem, 0) != ERR_OK)
+    coap_log_warn("coap_io_lwip_init: Failed to set up semaphore\n");
+#endif /* NO_SYS == 0 */
+}
+
+void
+coap_io_lwip_cleanup(void) {
+#if NO_SYS == 0
+  sys_sem_free(&coap_io_timeout_sem);
+#endif /* NO_SYS == 0 */
+}
+
 void
 coap_io_process_timeout(void *arg) {
-  coap_context_t *context = (coap_context_t *)arg;
-  coap_tick_t before;
-  unsigned int num_sockets;
-  unsigned int timeout;
-
-  coap_ticks(&before);
-  timeout = coap_io_prepare_io(context, NULL, 0, &num_sockets, before);
-  if (context->timer_configured) {
-    sys_untimeout(coap_io_process_timeout, (void *)context);
-    context->timer_configured = 0;
-  }
-  if (timeout == 0) {
-    /* Garbage collect 1 sec hence */
-    timeout = 1000;
-  }
-#ifdef COAP_DEBUG_WAKEUP_TIMES
-  coap_log_info("****** Next wakeup msecs %u (1)\n",
-                timeout);
-#endif /* COAP_DEBUG_WAKEUP_TIMES */
-  sys_timeout(timeout, coap_io_process_timeout, context);
-  context->timer_configured = 1;
+  (void)arg;
+#if NO_SYS == 0
+  sys_sem_signal(&coap_io_timeout_sem);
+#endif /* NO_SYS == 0 */
 }
 
 int
-coap_io_process(coap_context_t *context, uint32_t timeout_ms) {
+coap_io_process(coap_context_t *ctx, uint32_t timeout_ms) {
+  int ret;
+
+  coap_lock_lock(ctx, return 0);
+  ret = coap_io_process_lkd(ctx, timeout_ms);
+  coap_lock_unlock(ctx);
+  return ret;
+}
+
+int
+coap_io_process_lkd(coap_context_t *context, uint32_t timeout_ms) {
   coap_tick_t before;
   coap_tick_t now;
   unsigned int num_sockets;
   unsigned int timeout;
 
+  coap_lock_check_locked(context);
   coap_ticks(&before);
-  timeout = coap_io_prepare_io(context, NULL, 0, &num_sockets, before);
-  if (timeout_ms != 0 && timeout_ms != COAP_IO_NO_WAIT &&
-      timeout > timeout_ms) {
+  timeout = coap_io_prepare_io_lkd(context, NULL, 0, &num_sockets, before);
+  if (timeout == 0 || (timeout_ms != COAP_IO_WAIT && timeout_ms < timeout))
     timeout = timeout_ms;
-  }
 
-  LOCK_TCPIP_CORE();
+  if (timeout_ms == COAP_IO_NO_WAIT)
+    timeout = 1;
+
+  coap_lock_invert(context,
+                   LOCK_TCPIP_CORE(),
+                   UNLOCK_TCPIP_CORE(); return 0);
 
   if (context->timer_configured) {
     sys_untimeout(coap_io_process_timeout, (void *)context);
     context->timer_configured = 0;
   }
-  if (timeout == 0) {
-    /* Garbage collect 1 sec hence */
-    timeout = 1000;
-  }
 #ifdef COAP_DEBUG_WAKEUP_TIMES
   coap_log_info("****** Next wakeup msecs %u (2)\n",
                 timeout);
 #endif /* COAP_DEBUG_WAKEUP_TIMES */
-  sys_timeout(timeout, coap_io_process_timeout, context);
-  context->timer_configured = 1;
+  if (timeout) {
+    sys_timeout(timeout, coap_io_process_timeout, context);
+    context->timer_configured = 1;
+  }
 
   UNLOCK_TCPIP_CORE();
 
   if (context->input_wait) {
-    context->input_wait(context->input_arg, timeout);
+    coap_lock_callback_release(context,
+                               context->input_wait(context->input_arg, timeout),
+                               return 0);
+#if NO_SYS == 0
+  } else {
+    coap_lock_callback_release(context,
+                               sys_arch_sem_wait(&coap_io_timeout_sem, timeout),
+                               return 0);
+#endif /* NO_SYS == 0 */
   }
 
-  LOCK_TCPIP_CORE();
+  coap_lock_invert(context,
+                   LOCK_TCPIP_CORE(),
+                   UNLOCK_TCPIP_CORE(); return 0);
 
   sys_check_timeouts();
 
@@ -178,8 +202,13 @@ coap_recvc(void *arg, struct udp_pcb *upcb, struct pbuf *p,
     if (!coap_pdu_parse(session->proto, p->payload, p->len, pdu)) {
       goto error;
     }
+    coap_lock_lock(session->context, return);
     coap_dispatch(session->context, session, pdu);
+    coap_lock_unlock(session->context);
   }
+#if NO_SYS == 0
+  sys_sem_signal(&coap_io_timeout_sem);
+#endif /* NO_SYS == 0 */
   coap_delete_pdu(pdu);
   return;
 
@@ -189,7 +218,7 @@ error:
    * https://rfc-editor.org/rfc/rfc7252#section-4.3 MAY send RST
    */
   if (session)
-    coap_send_rst(session, pdu);
+    coap_send_rst_lkd(session, pdu);
   coap_delete_pdu(pdu);
   return;
 }
@@ -202,7 +231,7 @@ coap_free_packet(coap_packet_t *packet) {
   coap_free_type(COAP_PACKET, packet);
 }
 
-/** Callback from lwIP when a package was received for a server.
+/** Callback from lwIP when a UDP packet was received for a server.
  *
  * The current implementation deals this to coap_dispatch immediately, but
  * other mechanisms (as storing the package in a queue and later fetching it
@@ -211,18 +240,18 @@ coap_free_packet(coap_packet_t *packet) {
  * It handles everything coap_io_do_io does on other implementations.
  */
 static void
-coap_recvs(void *arg, struct udp_pcb *upcb, struct pbuf *p,
-           const ip_addr_t *addr, u16_t port) {
+coap_udp_recvs(void *arg, struct udp_pcb *upcb, struct pbuf *p,
+               const ip_addr_t *addr, u16_t port) {
   coap_endpoint_t *ep = (coap_endpoint_t *)arg;
   coap_pdu_t *pdu = NULL;
   coap_session_t *session = NULL;
   coap_tick_t now;
-  coap_packet_t *packet;
+  coap_packet_t *packet = NULL;
   int result = -1;
 
   if (p->len < 4) {
     /* Minimum size of CoAP header - ignore runt */
-    return;
+    goto error_free_pbuf;
   }
 
   packet = coap_malloc_type(COAP_PACKET, sizeof(coap_packet_t));
@@ -242,9 +271,10 @@ coap_recvs(void *arg, struct udp_pcb *upcb, struct pbuf *p,
 
   coap_ticks(&now);
 
+  coap_lock_lock(ep->context, goto error_free_pbuf);
   session = coap_endpoint_get_session(ep, packet, now);
   if (!session)
-    goto error;
+    goto error_free_pbuf;
   LWIP_ASSERT("Proto not supported for LWIP", COAP_PROTO_NOT_RELIABLE(session->proto));
 
   coap_log_debug("*  %s: lwip:  recv %4d bytes\n",
@@ -271,17 +301,25 @@ coap_recvs(void *arg, struct udp_pcb *upcb, struct pbuf *p,
 
   coap_delete_pdu(pdu);
   coap_free_packet(packet);
+  coap_lock_unlock(ep->context);
+#if NO_SYS == 0
+  sys_sem_signal(&coap_io_timeout_sem);
+#endif /* NO_SYS == 0 */
   return;
 
+error_free_pbuf:
+  pbuf_free(p);
+
 error:
   /*
    * https://rfc-editor.org/rfc/rfc7252#section-4.2 MUST send RST
    * https://rfc-editor.org/rfc/rfc7252#section-4.3 MAY send RST
    */
-  if (session)
-    coap_send_rst(session, pdu);
+  if (session && pdu)
+    coap_send_rst_lkd(session, pdu);
   coap_delete_pdu(pdu);
   coap_free_packet(packet);
+  coap_lock_unlock(ep->context);
   return;
 }
 
@@ -304,7 +342,7 @@ coap_socket_send_pdu(coap_socket_t *sock, coap_session_t *session,
     pbuf = pbuf_clone(PBUF_TRANSPORT, PBUF_RAM, pdu->pbuf);
     if (pbuf == NULL)
       return -1;
-    err = udp_sendto(sock->pcb, pbuf, &session->addr_info.remote.addr,
+    err = udp_sendto(sock->udp_pcb, pbuf, &session->addr_info.remote.addr,
                      session->addr_info.remote.port);
     pbuf_free(pbuf);
     if (err < 0)
@@ -319,7 +357,7 @@ coap_socket_send_pdu(coap_socket_t *sock, coap_session_t *session,
  *         -1 Error error in errno).
  */
 ssize_t
-coap_socket_send(coap_socket_t *sock, const coap_session_t *session,
+coap_socket_send(coap_socket_t *sock, coap_session_t *session,
                  const uint8_t *data, size_t data_len) {
   struct pbuf *pbuf;
   int err;
@@ -330,16 +368,26 @@ coap_socket_send(coap_socket_t *sock, const coap_session_t *session,
       return -1;
     memcpy(pbuf->payload, data, data_len);
 
-    LOCK_TCPIP_CORE();
+    coap_lock_invert(session->context,
+                     LOCK_TCPIP_CORE(),
+                     UNLOCK_TCPIP_CORE(); return -1);
 
-    err = udp_sendto(sock->pcb, pbuf, &session->addr_info.remote.addr,
+    err = udp_sendto(sock->udp_pcb, pbuf, &session->addr_info.remote.addr,
                      session->addr_info.remote.port);
 
     UNLOCK_TCPIP_CORE();
 
     pbuf_free(pbuf);
-    if (err < 0)
+    if (err < 0) {
+      if (err == ERR_RTE) {
+        coap_log_warn("** %s: udp_send: Packet not routable\n",
+                      coap_session_str(session));
+      } else {
+        coap_log_warn("** %s: udp_send: error %d\n",
+                      coap_session_str(session), err);
+      }
       return -1;
+    }
   }
   return data_len;
 }
@@ -352,19 +400,19 @@ coap_socket_bind_udp(coap_socket_t *sock,
   int err;
   coap_address_t l_listen = *listen_addr;
 
-  sock->pcb = udp_new_ip_type(IPADDR_TYPE_ANY);
-  if (sock->pcb == NULL)
+  sock->udp_pcb = udp_new_ip_type(IPADDR_TYPE_ANY);
+  if (sock->udp_pcb == NULL)
     return 0;
 
 #if LWIP_IPV6 && LWIP_IPV4
   if (l_listen.addr.type == IPADDR_TYPE_V6)
     l_listen.addr.type = IPADDR_TYPE_ANY;
 #endif /* LWIP_IPV6 && LWIP_IPV4 */
-  udp_recv(sock->pcb, coap_recvs, (void *)sock->endpoint);
-  err = udp_bind(sock->pcb, &l_listen.addr, l_listen.port);
+  udp_recv(sock->udp_pcb, coap_udp_recvs, (void *)sock->endpoint);
+  err = udp_bind(sock->udp_pcb, &l_listen.addr, l_listen.port);
   if (err) {
-    udp_remove(sock->pcb);
-    sock->pcb = NULL;
+    udp_remove(sock->udp_pcb);
+    sock->udp_pcb = NULL;
   }
   *bound_addr = l_listen;
   return err ? 0 : 1;
@@ -387,7 +435,9 @@ coap_socket_connect_udp(coap_socket_t *sock,
   (void)local_addr;
   (void)remote_addr;
 
-  LOCK_TCPIP_CORE();
+  coap_lock_invert(sock->session->context,
+                   LOCK_TCPIP_CORE(),
+                   goto err_unlock);
 
   pcb = udp_new();
 
@@ -413,9 +463,9 @@ coap_socket_connect_udp(coap_socket_t *sock,
   pcb->local_ip.type = pcb->remote_ip.type;
 #endif /* LWIP_IPV6 && LWIP_IPV4 */
 
-  sock->pcb = pcb;
+  sock->udp_pcb = pcb;
 
-  udp_recv(sock->pcb, coap_recvc, (void *)sock->session);
+  udp_recv(sock->udp_pcb, coap_recvc, (void *)sock->session);
 
   UNLOCK_TCPIP_CORE();
 
@@ -431,6 +481,77 @@ err_unlock:
 #endif /* ! COAP_CLIENT_SUPPORT */
 
 #if ! COAP_DISABLE_TCP
+
+#include <lwip/tcp.h>
+
+static void
+do_tcp_err(void *arg, err_t err) {
+  coap_session_t *session = (coap_session_t *)arg;
+
+  (void)err;
+
+  coap_handle_event_lkd(session->context, COAP_EVENT_TCP_FAILED, session);
+  /*
+   * as per tcp_err() documentation, the corresponding pcb is already freed
+   * when this callback is called.  So, stop a double free when
+   * coap_session_disconnected_lkd() eventually coap_socket_close() is called.
+   */
+  session->sock.tcp_pcb = NULL;
+  coap_session_disconnected_lkd(session, COAP_NACK_NOT_DELIVERABLE);
+}
+
+/** Callback from lwIP when a TCP packet is received.
+ *
+ * The current implementation invokes coap_read_session() to do the bulk of the
+ * work.
+ */
+static err_t
+coap_tcp_recv(void *arg, struct tcp_pcb *tpcb, struct pbuf *p, err_t err) {
+  coap_session_t *session = (coap_session_t *)arg;
+  coap_socket_t *sock = &session->sock;
+  coap_tick_t now;
+
+  (void)tpcb;
+  if (p == NULL) {
+    /* remote host closed connection */
+    tcp_arg(sock->tcp_pcb, NULL);
+    tcp_recv(sock->tcp_pcb, NULL);
+    tcp_close(sock->tcp_pcb);
+    sock->tcp_pcb = NULL;
+    coap_session_disconnected_lkd(session, COAP_NACK_NOT_DELIVERABLE);
+    return ERR_OK;
+  } else if (err != ERR_OK) {
+    /* cleanup, for unknown reason */
+    if (p != NULL) {
+      pbuf_free(p);
+    }
+    return err;
+  }
+
+  sock->p = p;
+  coap_ticks(&now);
+  coap_read_session(session->context, session, now);
+  return ERR_OK;
+}
+
+#if COAP_CLIENT_SUPPORT
+
+static err_t
+do_tcp_connected(void *arg, struct tcp_pcb *tpcb, err_t err) {
+  coap_session_t *session = (coap_session_t *)arg;
+  coap_tick_t now;
+
+  if (err)
+    return err;
+  session->sock.flags |= COAP_SOCKET_CONNECTED;
+  session->addr_info.local.addr = tpcb->local_ip;
+  session->addr_info.local.port = tpcb->local_port;
+  tcp_recv(tpcb, coap_tcp_recv);
+  coap_ticks(&now);
+  coap_connect_session(session, now);
+  return ERR_OK;
+}
+
 int
 coap_socket_connect_tcp1(coap_socket_t *sock,
                          const coap_address_t *local_if,
@@ -438,13 +559,45 @@ coap_socket_connect_tcp1(coap_socket_t *sock,
                          int default_port,
                          coap_address_t *local_addr,
                          coap_address_t *remote_addr) {
-  (void)sock;
-  (void)local_if;
-  (void)server;
-  (void)default_port;
+  coap_address_t connect_addr;
+  err_t err;
+
   (void)local_addr;
   (void)remote_addr;
-  return 0;
+
+  sock->flags &= ~(COAP_SOCKET_WANT_CONNECT | COAP_SOCKET_CONNECTED);
+
+  sock->tcp_pcb = tcp_new_ip_type(IPADDR_TYPE_ANY);
+  if (sock->tcp_pcb == NULL)
+    return 0;
+
+  tcp_arg(sock->tcp_pcb, sock->session);
+  tcp_recv(sock->tcp_pcb, coap_tcp_recv);
+  tcp_err(sock->tcp_pcb, do_tcp_err);
+  if (local_if) {
+    coap_address_t l_local_if = *local_if;
+#if LWIP_IPV6 && LWIP_IPV4
+    if (l_local_if.addr.type == IPADDR_TYPE_V6)
+      l_local_if.addr.type = IPADDR_TYPE_ANY;
+#endif /* LWIP_IPV6 && LWIP_IPV4 */
+    err = tcp_bind(sock->tcp_pcb, &l_local_if.addr, l_local_if.port);
+    if (err != ERR_OK) {
+      tcp_arg(sock->tcp_pcb, NULL);
+      tcp_recv(sock->tcp_pcb, NULL);
+      tcp_close(sock->tcp_pcb);
+      sock->tcp_pcb = NULL;
+      return 0;
+    }
+  }
+  coap_address_copy(&connect_addr, server);
+  if (connect_addr.port == 0)
+    connect_addr.port = htons(default_port);
+
+  err = tcp_connect(sock->tcp_pcb, &connect_addr.addr, connect_addr.port,
+                    do_tcp_connected);
+  if (err == ERR_OK)
+    sock->flags |= COAP_SOCKET_WANT_CONNECT | COAP_SOCKET_CONNECTED;
+  return err ? 0 : 1;
 }
 
 int
@@ -454,55 +607,190 @@ coap_socket_connect_tcp2(coap_socket_t *sock,
   (void)sock;
   (void)local_addr;
   (void)remote_addr;
-  return 0;
+
+  sock->flags &= ~(COAP_SOCKET_WANT_CONNECT | COAP_SOCKET_CAN_CONNECT);
+  return 1;
+}
+#endif /* COAP_CLIENT_SUPPORT */
+
+#if COAP_SERVER_SUPPORT
+
+static err_t
+do_tcp_accept(void *arg, struct tcp_pcb *newpcb, err_t err) {
+  coap_endpoint_t *endpoint = arg;
+  coap_session_t *session;
+  coap_tick_t now;
+  err_t ret_err = ERR_OK;
+
+  if ((err != ERR_OK) || (newpcb == NULL)) {
+    return ERR_VAL;
+  }
+  coap_ticks(&now);
+
+  session = coap_new_server_session(endpoint->context, endpoint, newpcb);
+
+  if (session) {
+    session->sock.tcp_pcb = newpcb;
+    session->last_rx_tx = now;
+    tcp_arg(newpcb, session);
+    tcp_setprio(newpcb, TCP_PRIO_MIN);
+    tcp_recv(newpcb, coap_tcp_recv);
+    tcp_err(newpcb, do_tcp_err);
+  } else {
+    ret_err = ERR_MEM;
+  }
+  return ret_err;
 }
 
 int
 coap_socket_bind_tcp(coap_socket_t *sock,
                      const coap_address_t *listen_addr,
                      coap_address_t *bound_addr) {
-  (void)sock;
-  (void)listen_addr;
-  (void)bound_addr;
-  return 0;
+  int err;
+  coap_address_t l_listen = *listen_addr;
+  struct tcp_pcb *tcp_pcb;
+
+  sock->tcp_pcb = tcp_new_ip_type(IPADDR_TYPE_ANY);
+  if (sock->tcp_pcb == NULL)
+    return 0;
+
+#if LWIP_IPV6 && LWIP_IPV4
+  if (l_listen.addr.type == IPADDR_TYPE_V6)
+    l_listen.addr.type = IPADDR_TYPE_ANY;
+#endif /* LWIP_IPV6 && LWIP_IPV4 */
+  tcp_arg(sock->tcp_pcb, sock->endpoint);
+  err = tcp_bind(sock->tcp_pcb, &l_listen.addr, l_listen.port);
+  if (err != ERR_OK) {
+    tcp_arg(sock->tcp_pcb, NULL);
+    tcp_recv(sock->tcp_pcb, NULL);
+    tcp_close(sock->tcp_pcb);
+    sock->tcp_pcb = NULL;
+    return 0;
+  } else {
+    tcp_pcb = tcp_listen(sock->tcp_pcb);
+    if (tcp_pcb) {
+      sock->tcp_pcb = tcp_pcb;
+      tcp_accept(sock->tcp_pcb, do_tcp_accept);
+    } else {
+      tcp_arg(sock->tcp_pcb, NULL);
+      tcp_recv(sock->tcp_pcb, NULL);
+      tcp_close(sock->tcp_pcb);
+      sock->tcp_pcb = NULL;
+      return 0;
+    }
+  }
+  *bound_addr = l_listen;
+  return err ? 0 : 1;
 }
 
 int
 coap_socket_accept_tcp(coap_socket_t *server,
                        coap_socket_t *new_client,
                        coap_address_t *local_addr,
-                       coap_address_t *remote_addr) {
+                       coap_address_t *remote_addr,
+                       void *extra) {
+  struct tcp_pcb *tcp_pcb = (struct tcp_pcb *)extra;
+
   (void)server;
-  (void)new_client;
-  (void)local_addr;
-  (void)remote_addr;
+
+  new_client->tcp_pcb = tcp_pcb;
+  local_addr->addr = tcp_pcb->local_ip;
+  local_addr->port = tcp_pcb->local_port;
+  remote_addr->addr = tcp_pcb->remote_ip;
+  remote_addr->port = tcp_pcb->remote_port;
+  return 1;
+}
+#endif /* COAP_SERVER_SUPPORT */
+
+/*
+ * strm
+ * return +ve Number of bytes written.
+ *         -1 Error error in errno).
+ */
+ssize_t
+coap_socket_write(coap_socket_t *sock, const uint8_t *data, size_t data_len) {
+  struct pbuf *pbuf;
+  int err;
+
+  pbuf = pbuf_alloc(PBUF_TRANSPORT, data_len, PBUF_RAM);
+  if (pbuf == NULL)
+    return -1;
+  memcpy(pbuf->payload, data, data_len);
+
+  coap_lock_invert(context,
+                   LOCK_TCPIP_CORE(),
+                   UNLOCK_TCPIP_CORE(); return 0);
+
+  err = tcp_write(sock->tcp_pcb, pbuf->payload, pbuf->len, 1);
+
+  UNLOCK_TCPIP_CORE();
+
+  pbuf_free(pbuf);
+  if (err < 0)
+    return -1;
+  return data_len;
+}
+
+/*
+ * strm
+ * return >=0 Number of bytes read.
+ *         -1 Error error in errno).
+ */
+ssize_t
+coap_socket_read(coap_socket_t *sock, uint8_t *data, size_t data_len) {
+  if (sock->p) {
+    if (data_len < sock->p->len) {
+      uint8_t *ptr = (uint8_t *)sock->p->payload;
+
+      /* Handle partial read of data request */
+      memcpy(data, sock->p->payload, data_len);
+      sock->p->payload = &ptr[data_len];
+      sock->p->len -= data_len;
+      return data_len;
+    } else {
+      data_len = sock->p->len;
+      memcpy(data, sock->p->payload, sock->p->len);
+      pbuf_free(sock->p);
+      sock->p = NULL;
+      return data_len;
+    }
+  }
   return 0;
 }
 #endif /* !COAP_DISABLE_TCP */
 
-ssize_t
-coap_socket_write(coap_socket_t *sock, const uint8_t *data, size_t data_len) {
-  (void)sock;
-  (void)data;
-  (void)data_len;
-  return -1;
-}
-
-ssize_t
-coap_socket_read(coap_socket_t *sock, uint8_t *data, size_t data_len) {
-  (void)sock;
-  (void)data;
-  (void)data_len;
-  return -1;
-}
-
 void
 coap_socket_close(coap_socket_t *sock) {
-  if (sock->pcb) {
-    LOCK_TCPIP_CORE();
-    udp_remove(sock->pcb);
+  if (sock->udp_pcb) {
+    if (sock->session) {
+      coap_lock_invert(sock->session->context,
+                       LOCK_TCPIP_CORE(),
+                       UNLOCK_TCPIP_CORE(); return);
+    } else {
+      LOCK_TCPIP_CORE();
+    }
+    udp_remove(sock->udp_pcb);
     UNLOCK_TCPIP_CORE();
+    sock->udp_pcb = NULL;
   }
-  sock->pcb = NULL;
+#if ! COAP_DISABLE_TCP
+  if (sock->tcp_pcb) {
+    tcp_arg(sock->tcp_pcb, NULL);
+#if COAP_SERVER_SUPPORT
+    if (!sock->endpoint)
+#endif /* COAP_SERVER_SUPPORT */
+      tcp_recv(sock->tcp_pcb, NULL);
+    if (sock->session) {
+      coap_lock_invert(sock->session->context,
+                       LOCK_TCPIP_CORE(),
+                       UNLOCK_TCPIP_CORE(); return);
+    } else {
+      LOCK_TCPIP_CORE();
+    }
+    tcp_close(sock->tcp_pcb);
+    UNLOCK_TCPIP_CORE();
+    sock->tcp_pcb = NULL;
+  }
+#endif /* !COAP_DISABLE_TCP */
   return;
 }
