@@ -43,7 +43,7 @@
  *      Jon Shallow <supjps-libcoap@jpshallow.com>
  */
 
-#include "coap3/coap_internal.h"
+#include "coap3/coap_libcoap_build.h"
 
 #include <stdio.h>
 
@@ -705,8 +705,8 @@ oscore_new_association(coap_session_t *session,
     size_t size;
     const uint8_t *data;
 
-    association->sent_pdu = coap_pdu_duplicate(sent_pdu, session,
-                                               token->length, token->s, NULL);
+    association->sent_pdu = coap_pdu_duplicate_lkd(sent_pdu, session,
+                                                   token->length, token->s, NULL);
     if (association->sent_pdu == NULL)
       goto error;
     if (coap_get_data(sent_pdu, &size, &data)) {
