@@ -2,7 +2,7 @@
 
 /* libcoap unit tests
  *
- * Copyright (C) 2021-2023 Jon Shallow <supjps-libcoap@jpshallow.com>
+ * Copyright (C) 2021-2024 Jon Shallow <supjps-libcoap@jpshallow.com>
  *
  * SPDX-License-Identifier: BSD-2-Clause
  *
@@ -21,6 +21,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+static coap_context_t *ctx; /* Holds the coap context for most tests */
+
 #define CHECK_SAME(a,b) \
   (sizeof((a)) == (b)->length && memcmp((a), (b)->s, (b)->length) == 0)
 
@@ -61,13 +63,11 @@ t_oscore_c_1_1(void) {
   const coap_str_const_t conf = { sizeof(conf_data)-1,
                                   (const uint8_t *)conf_data
                                 };
-  coap_context_t ctx[1];
   coap_oscore_conf_t *oscore_conf;
   cose_encrypt0_t cose[1];
   uint8_t nonce_buffer[13];
   coap_bin_const_t nonce = { 13, nonce_buffer };
 
-  memset(&ctx, 0, sizeof(ctx));
   oscore_conf = coap_new_oscore_conf(conf, NULL, NULL, 0);
   FailIf_CU_ASSERT_PTR_NOT_NULL(oscore_conf);
   coap_context_oscore_server(ctx, oscore_conf);
@@ -125,13 +125,11 @@ t_oscore_c_1_2(void) {
   const coap_str_const_t conf = { sizeof(conf_data)-1,
                                   (const uint8_t *)conf_data
                                 };
-  coap_context_t ctx[1];
   coap_oscore_conf_t *oscore_conf;
   cose_encrypt0_t cose[1];
   uint8_t nonce_buffer[13];
   coap_bin_const_t nonce = { 13, nonce_buffer };
 
-  memset(&ctx, 0, sizeof(ctx));
   oscore_conf = coap_new_oscore_conf(conf, NULL, NULL, 0);
   FailIf_CU_ASSERT_PTR_NOT_NULL(oscore_conf);
   coap_context_oscore_server(ctx, oscore_conf);
@@ -188,13 +186,11 @@ t_oscore_c_2_1(void) {
   const coap_str_const_t conf = { sizeof(conf_data)-1,
                                   (const uint8_t *)conf_data
                                 };
-  coap_context_t ctx[1];
   coap_oscore_conf_t *oscore_conf;
   cose_encrypt0_t cose[1];
   uint8_t nonce_buffer[13];
   coap_bin_const_t nonce = { 13, nonce_buffer };
 
-  memset(&ctx, 0, sizeof(ctx));
   oscore_conf = coap_new_oscore_conf(conf, NULL, NULL, 0);
   FailIf_CU_ASSERT_PTR_NOT_NULL(oscore_conf);
   coap_context_oscore_server(ctx, oscore_conf);
@@ -251,13 +247,11 @@ t_oscore_c_2_2(void) {
   const coap_str_const_t conf = { sizeof(conf_data)-1,
                                   (const uint8_t *)conf_data
                                 };
-  coap_context_t ctx[1];
   coap_oscore_conf_t *oscore_conf;
   cose_encrypt0_t cose[1];
   uint8_t nonce_buffer[13];
   coap_bin_const_t nonce = { 13, nonce_buffer };
 
-  memset(&ctx, 0, sizeof(ctx));
   oscore_conf = coap_new_oscore_conf(conf, NULL, NULL, 0);
   FailIf_CU_ASSERT_PTR_NOT_NULL(oscore_conf);
   coap_context_oscore_server(ctx, oscore_conf);
@@ -316,13 +310,11 @@ t_oscore_c_3_1(void) {
   const coap_str_const_t conf = { sizeof(conf_data)-1,
                                   (const uint8_t *)conf_data
                                 };
-  coap_context_t ctx[1];
   coap_oscore_conf_t *oscore_conf;
   cose_encrypt0_t cose[1];
   uint8_t nonce_buffer[13];
   coap_bin_const_t nonce = { 13, nonce_buffer };
 
-  memset(&ctx, 0, sizeof(ctx));
   oscore_conf = coap_new_oscore_conf(conf, NULL, NULL, 0);
   FailIf_CU_ASSERT_PTR_NOT_NULL(oscore_conf);
   coap_context_oscore_server(ctx, oscore_conf);
@@ -381,13 +373,11 @@ t_oscore_c_3_2(void) {
   const coap_str_const_t conf = { sizeof(conf_data)-1,
                                   (const uint8_t *)conf_data
                                 };
-  coap_context_t ctx[1];
   coap_oscore_conf_t *oscore_conf;
   cose_encrypt0_t cose[1];
   uint8_t nonce_buffer[13];
   coap_bin_const_t nonce = { 13, nonce_buffer };
 
-  memset(&ctx, 0, sizeof(ctx));
   oscore_conf = coap_new_oscore_conf(conf, NULL, NULL, 0);
   FailIf_CU_ASSERT_PTR_NOT_NULL(oscore_conf);
   coap_context_oscore_server(ctx, oscore_conf);
@@ -437,14 +427,12 @@ t_oscore_c_4(void) {
   const coap_str_const_t conf = { sizeof(conf_data)-1,
                                   (const uint8_t *)conf_data
                                 };
-  coap_context_t ctx[1];
   coap_oscore_conf_t *oscore_conf;
   int result;
   coap_pdu_t *pdu = coap_pdu_init(0, 0, 0, COAP_DEFAULT_MTU);
   coap_pdu_t *osc_pdu = NULL;
   coap_session_t *session = NULL;
 
-  memset(&ctx, 0, sizeof(ctx));
   FailIf_CU_ASSERT_PTR_NOT_NULL(pdu);
 
   oscore_conf = coap_new_oscore_conf(conf, NULL, NULL, 20);
@@ -459,6 +447,7 @@ t_oscore_c_4(void) {
   session = coap_malloc_type(COAP_SESSION, sizeof(coap_session_t));
   FailIf_CU_ASSERT_PTR_NOT_NULL(session);
   memset(session, 0, sizeof(coap_session_t));
+  session->context = ctx;
   session->proto = COAP_PROTO_UDP;
   session->type = COAP_SESSION_TYPE_CLIENT;
   session->recipient_ctx = ctx->p_osc_ctx->recipient_chain;
@@ -504,14 +493,12 @@ t_oscore_c_5(void) {
   const coap_str_const_t conf = { sizeof(conf_data)-1,
                                   (const uint8_t *)conf_data
                                 };
-  coap_context_t ctx[1];
   coap_oscore_conf_t *oscore_conf;
   int result;
   coap_pdu_t *pdu = coap_pdu_init(0, 0, 0, COAP_DEFAULT_MTU);
   coap_pdu_t *osc_pdu = NULL;
   coap_session_t *session = NULL;
 
-  memset(&ctx, 0, sizeof(ctx));
   FailIf_CU_ASSERT_PTR_NOT_NULL(pdu);
 
   oscore_conf = coap_new_oscore_conf(conf, NULL, NULL, 20);
@@ -526,6 +513,7 @@ t_oscore_c_5(void) {
   session = coap_malloc_type(COAP_SESSION, sizeof(coap_session_t));
   FailIf_CU_ASSERT_PTR_NOT_NULL(session);
   memset(session, 0, sizeof(coap_session_t));
+  session->context = ctx;
   session->proto = COAP_PROTO_UDP;
   session->type = COAP_SESSION_TYPE_CLIENT;
   session->recipient_ctx = ctx->p_osc_ctx->recipient_chain;
@@ -573,14 +561,12 @@ t_oscore_c_6(void) {
   const coap_str_const_t conf = { sizeof(conf_data)-1,
                                   (const uint8_t *)conf_data
                                 };
-  coap_context_t ctx[1];
   coap_oscore_conf_t *oscore_conf;
   int result;
   coap_pdu_t *pdu = coap_pdu_init(0, 0, 0, COAP_DEFAULT_MTU);
   coap_pdu_t *osc_pdu = NULL;
   coap_session_t *session = NULL;
 
-  memset(&ctx, 0, sizeof(ctx));
   FailIf_CU_ASSERT_PTR_NOT_NULL(pdu);
 
   oscore_conf = coap_new_oscore_conf(conf, NULL, NULL, 20);
@@ -595,6 +581,7 @@ t_oscore_c_6(void) {
   session = coap_malloc_type(COAP_SESSION, sizeof(coap_session_t));
   FailIf_CU_ASSERT_PTR_NOT_NULL(session);
   memset(session, 0, sizeof(coap_session_t));
+  session->context = ctx;
   session->proto = COAP_PROTO_UDP;
   session->type = COAP_SESSION_TYPE_CLIENT;
   session->recipient_ctx = ctx->p_osc_ctx->recipient_chain;
@@ -651,7 +638,6 @@ t_oscore_c_7(void) {
   const coap_str_const_t conf = { sizeof(conf_data)-1,
                                   (const uint8_t *)conf_data
                                 };
-  coap_context_t ctx[1];
   coap_oscore_conf_t *oscore_conf;
   int result;
   coap_pdu_t *incoming_pdu = coap_pdu_init(0, 0, 0, COAP_DEFAULT_MTU);
@@ -659,7 +645,6 @@ t_oscore_c_7(void) {
   coap_pdu_t *osc_pdu = NULL;
   coap_session_t *session = NULL;
 
-  memset(&ctx, 0, sizeof(ctx));
   FailIf_CU_ASSERT_PTR_NOT_NULL(incoming_pdu);
   FailIf_CU_ASSERT_PTR_NOT_NULL(pdu);
 
@@ -679,6 +664,7 @@ t_oscore_c_7(void) {
   session = coap_malloc_type(COAP_SESSION, sizeof(coap_session_t));
   FailIf_CU_ASSERT_PTR_NOT_NULL(session);
   memset(session, 0, sizeof(coap_session_t));
+  session->context = ctx;
   session->proto = COAP_PROTO_UDP;
   session->type = COAP_SESSION_TYPE_SERVER;
   session->recipient_ctx = ctx->p_osc_ctx->recipient_chain;
@@ -758,7 +744,6 @@ t_oscore_c_7_2(void) {
   const coap_str_const_t conf = { sizeof(conf_data)-1,
                                   (const uint8_t *)conf_data
                                 };
-  coap_context_t ctx[1];
   coap_oscore_conf_t *oscore_conf;
   int result;
   coap_pdu_t *outgoing_pdu = coap_pdu_init(0, 0, 0, COAP_DEFAULT_MTU);
@@ -766,7 +751,6 @@ t_oscore_c_7_2(void) {
   coap_pdu_t *osc_pdu = NULL;
   coap_session_t *session = NULL;
 
-  memset(&ctx, 0, sizeof(ctx));
   FailIf_CU_ASSERT_PTR_NOT_NULL(outgoing_pdu);
   FailIf_CU_ASSERT_PTR_NOT_NULL(incoming_pdu);
 
@@ -785,6 +769,7 @@ t_oscore_c_7_2(void) {
   session = coap_malloc_type(COAP_SESSION, sizeof(coap_session_t));
   FailIf_CU_ASSERT_PTR_NOT_NULL(session);
   memset(session, 0, sizeof(coap_session_t));
+  session->context = ctx;
   session->proto = COAP_PROTO_UDP;
   session->type = COAP_SESSION_TYPE_CLIENT;
   session->recipient_ctx = ctx->p_osc_ctx->recipient_chain;
@@ -865,7 +850,6 @@ t_oscore_c_8(void) {
   const coap_str_const_t conf = { sizeof(conf_data)-1,
                                   (const uint8_t *)conf_data
                                 };
-  coap_context_t ctx[1];
   coap_oscore_conf_t *oscore_conf;
   int result;
   coap_pdu_t *incoming_pdu = coap_pdu_init(0, 0, 0, COAP_DEFAULT_MTU);
@@ -873,7 +857,6 @@ t_oscore_c_8(void) {
   coap_pdu_t *osc_pdu = NULL;
   coap_session_t *session = NULL;
 
-  memset(&ctx, 0, sizeof(ctx));
   FailIf_CU_ASSERT_PTR_NOT_NULL(incoming_pdu);
   FailIf_CU_ASSERT_PTR_NOT_NULL(pdu);
 
@@ -893,6 +876,7 @@ t_oscore_c_8(void) {
   session = coap_malloc_type(COAP_SESSION, sizeof(coap_session_t));
   FailIf_CU_ASSERT_PTR_NOT_NULL(session);
   memset(session, 0, sizeof(coap_session_t));
+  session->context = ctx;
   session->proto = COAP_PROTO_UDP;
   session->type = COAP_SESSION_TYPE_SERVER;
   session->recipient_ctx = ctx->p_osc_ctx->recipient_chain;
@@ -973,7 +957,6 @@ t_oscore_c_8_2(void) {
   const coap_str_const_t conf = { sizeof(conf_data)-1,
                                   (const uint8_t *)conf_data
                                 };
-  coap_context_t ctx[1];
   coap_oscore_conf_t *oscore_conf;
   int result;
   coap_pdu_t *outgoing_pdu = coap_pdu_init(0, 0, 0, COAP_DEFAULT_MTU);
@@ -981,7 +964,6 @@ t_oscore_c_8_2(void) {
   coap_pdu_t *osc_pdu = NULL;
   coap_session_t *session = NULL;
 
-  memset(&ctx, 0, sizeof(ctx));
   FailIf_CU_ASSERT_PTR_NOT_NULL(outgoing_pdu);
   FailIf_CU_ASSERT_PTR_NOT_NULL(incoming_pdu);
 
@@ -1000,6 +982,7 @@ t_oscore_c_8_2(void) {
   session = coap_malloc_type(COAP_SESSION, sizeof(coap_session_t));
   FailIf_CU_ASSERT_PTR_NOT_NULL(session);
   memset(session, 0, sizeof(coap_session_t));
+  session->context = ctx;
   session->proto = COAP_PROTO_UDP;
   session->type = COAP_SESSION_TYPE_CLIENT;
   session->recipient_ctx = ctx->p_osc_ctx->recipient_chain;
@@ -1049,11 +1032,25 @@ fail:
  ** initialization
  ************************************************************************/
 
+static int
+t_oscore_tests_create(void) {
+  ctx = coap_new_context(NULL);
+
+  return (ctx == NULL);
+}
+
+static int
+t_oscore_tests_remove(void) {
+  coap_free_context(ctx);
+  return 0;
+}
+
 CU_pSuite
 t_init_oscore_tests(void) {
   CU_pSuite suite[5];
 
-  suite[0] = CU_add_suite("RFC8613 Appendix C OSCORE tests", NULL, NULL);
+  suite[0] = CU_add_suite("RFC8613 Appendix C OSCORE tests",
+                          t_oscore_tests_create, t_oscore_tests_remove);
   if (!suite[0]) {                        /* signal error */
     fprintf(stderr, "W: cannot add OSCORE test suite (%s)\n",
             CU_get_error_msg());
